% Example code for cnv2mat
% Michael Hirsch * 20/07/2010

clear;
rand('state', 0);

% Following notation used:
% y = f * x = Fx
%   = x * f = Xf
% sy = size(y); sx = size(x); sf = size(f);
% It holds: sy = sx - sf + 1

x = mean(double(imread('lena.png'))/255,3);     % load image
f = rand(30);                                   % generate PSF
f = f/sum(f(:));                                % normalize due to 
                                                % energy conservation
sf = size(f);
shape = 'valid';

% Convolution, i.e. computes y = Xf
% Code supports color images !
X = cnv2mat(x, sf, shape);
y = X * f;

% Transpose, i.e. computes y = X'f
f_transpose = X' * y;

% Test against 'conv2' of matlab
y_matlab = conv2(x(:,:,1), f, shape);

% Plotting
subplot(131)
imagesc(y); axis image; 
title('Result of cnv2mat')
subplot(132)
imagesc(y_matlab); colormap('gray'); axis image;
title('Result of matlab conv2')
subplot(133)
imagesc(y(:,:,1)-y_matlab); axis image;
title('Difference image')
colorbar()




